import { createTransformStream } from "@datastream/core";
import _ajv from "ajv/dist/2020.js";
import formats from "ajv-formats";
import formatsDraft2019 from "ajv-formats-draft2019";
import uriResolver from "fast-uri";
const Ajv = _ajv.default;
const ajvDefaults = {
  strict: true,
  coerceTypes: "array",
  allErrors: true,
  useDefaults: "empty",
  uriResolver
};
const jsonSchemaValidateDefaults = {
  idxStart: 0,
  key: "ajvErrors",
  language: "en",
  availableLanguages: void 0
};
const validateStream = (chunkSchema, config, options) => {
  const { idxStart, key } = {
    ...jsonSchemaValidateDefaults,
    ...config
  };
  if (typeof chunkSchema !== "function") {
    const ajv = new Ajv(ajvDefaults);
    chunkSchema = ajv.compile(chunkSchema);
  }
  const value = {};
  let idx = idxStart - 1;
  const transform = (chunk) => {
    idx += 1;
    const chunkValid = chunkSchema(chunk);
    console.log({ chunkValid });
    if (!chunkValid) {
      for (const error of chunkSchema.errors) {
        const { id, keys, message } = processError(error);
        if (!value[id]) {
          value[id] = { id, keys, message, idx: [] };
        }
        value[id].idx.push(idx);
      }
    }
    return chunk;
  };
  const stream = createTransformStream(transform, options);
  stream.result = () => ({ key, value });
  return stream;
};
const processError = (error) => {
  const message = error.message || "";
  let id = error.schemaPath;
  let keys = [];
  if (error.keyword === "errorMessage") {
    error.params.errors.forEach((error2) => {
      const value = makeKeys(error2);
      if (value)
        keys.push(value);
    });
    keys = [...new Set(keys.sort())];
  } else {
    keys.push(makeKeys(error));
  }
  if (!error.instancePath && keys.length) {
    id += `/${keys.join("|")}`;
  }
  return { id, keys, message };
};
const makeKeys = (error) => {
  return error.params.missingProperty || error.params.additionalProperty || error.instancePath.replace("/", "");
};
var validate_default = validateStream;
export {
  validate_default as default,
  validateStream
};
